@@ -35,7 +35,9 @@
 
 #include <tiledb/common/logger_public.h>
 #include "tiledb.h"
+#include "tiledb/common/common.h"
 #include "tiledb/sm/array/array.h"
+#include "tiledb/sm/cpp_api/tiledb"
 #include "tiledb/sm/enums/layout.h"
 #include "tiledb/sm/enums/serialization_type.h"
 #include "tiledb/sm/stats/stats.h"
@@ -79,7 +81,7 @@ static tiledb::sm::stats::Stats g_helper_stats("test");
 
 // A dummy `Logger` instance. This is useful for constructing
 // objects that require a parent `Logger` object.
-tdb_shared_ptr<Logger> g_helper_logger(void);
+shared_ptr<Logger> g_helper_logger(void);
 
 // For easy reference
 typedef std::pair<tiledb_filter_type_t, int> Compressor;
@@ -157,6 +159,18 @@ template <class T>
 void check_subarray(
     tiledb::sm::Subarray& subarray, const SubarrayRanges<T>& ranges);
 
+template <class T>
+void check_subarray(
+    tiledb::Subarray& subarray, const SubarrayRanges<T>& ranges);
+
+template <class T>
+void check_subarray_equiv(
+    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
+
+template <class T>
+bool subarray_equiv(
+    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
+
 /**
  * Closes an array.
  *
@@ -315,6 +329,44 @@ void create_subarray(
     tiledb::sm::Subarray* subarray,
     bool coalesce_ranges = false);
 
+/**
+ * Creates a capi subarray for the input array.
+ *
+ * @tparam T The datatype of the subarray domain.
+ * @param array The input array.
+ * @param ranges The ranges of the subarray to be created.
+ * @param layout The layout of the subarray.
+ * @param subarray The subarray to be set.
+ * @param coalesce_ranges Whether the subarray should coalesce ranges.
+ */
+template <class T>
+void create_subarray(
+    tiledb_ctx_t* ctx,
+    tiledb::sm::Array* array,
+    const SubarrayRanges<T>& ranges,
+    tiledb::sm::Layout layout,
+    tiledb_subarray_t** subarray,
+    bool coalesce_ranges = false);
+
+/**
+ * Creates a c++ api subarray for the input array.
+ *
+ * @tparam T The datatype of the subarray domain.
+ * @param array The input array.
+ * @param ranges The ranges of the subarray to be created.
+ * @param layout The layout of the subarray.
+ * @param subarray The subarray to be set.
+ * @param coalesce_ranges Whether the subarray should coalesce ranges.
+ */
+template <class T>
+void create_subarray(
+    const tiledb::Context* ctx,
+    const tiledb::Array* array,
+    const SubarrayRanges<T>& ranges,
+    tiledb::sm::Layout layout,
+    tiledb::Subarray** subarray,
+    bool coalesce_ranges = false);
+
 /**
  * Helper method that creates a TileDB context and a VFS object.
  *
@@ -625,6 +677,21 @@ void read_array(
  */
 int32_t num_fragments(const std::string& array_name);
 
+/**
+ * Creates a random string of length l.
+ */
+std::string random_string(const uint64_t l);
+
+/**
+ * Gets the fragments directory from the array directory.
+ */
+std::string get_fragment_dir(std::string array_dir);
+
+/**
+ * Gets the commit directory from the array directory.
+ */
+std::string get_commit_dir(std::string array_dir);
+
 }  // End of namespace test
 
 }  // End of namespace tiledb
