@@ -32,7 +32,7 @@
 
 #include "test/src/helpers.h"
 #include "test/src/vfs_helpers.h"
-#include "tiledb/common/heap_memory.h"
+#include "tiledb/common/common.h"
 #include "tiledb/sm/array_schema/tile_domain.h"
 #include "tiledb/sm/c_api/tiledb_struct_def.h"
 #include "tiledb/sm/query/read_cell_slab_iter.h"
@@ -72,13 +72,13 @@ struct ReadCellSlabIterFx {
       const std::vector<std::vector<uint64_t>>& c_result_cell_slabs);
   template <class T>
   void create_result_space_tiles(
-      const std::vector<tdb_shared_ptr<FragmentMetadata>>& fragments,
-      const Domain* dom,
+      const std::vector<shared_ptr<FragmentMetadata>>& fragments,
+      const Domain& dom,
       const NDRange& dom_ndrange,
       Layout layout,
       const std::vector<NDRange>& domain_slices,
       const std::vector<std::vector<uint8_t>>& tile_coords,
-      std::map<const T*, ResultSpaceTile<T>>* result_space_tiles);
+      std::map<const T*, ResultSpaceTile<T>>& result_space_tiles);
 };
 
 ReadCellSlabIterFx::ReadCellSlabIterFx()
@@ -137,15 +137,15 @@ void ReadCellSlabIterFx::check_iter(
 
 template <class T>
 void ReadCellSlabIterFx::create_result_space_tiles(
-    const std::vector<tdb_shared_ptr<FragmentMetadata>>& fragments,
-    const Domain* dom,
+    const std::vector<shared_ptr<FragmentMetadata>>& fragments,
+    const Domain& dom,
     const NDRange& dom_ndrange,
     Layout layout,
     const std::vector<NDRange>& domain_slices,
     const std::vector<std::vector<uint8_t>>& tile_coords,
-    std::map<const T*, ResultSpaceTile<T>>* result_space_tiles) {
-  auto domain = dom->domain();
-  const auto& tile_extents = dom->tile_extents();
+    std::map<const T*, ResultSpaceTile<T>>& result_space_tiles) {
+  auto domain = dom.domain();
+  const auto& tile_extents = dom.tile_extents();
   std::vector<TileDomain<T>> frag_tile_domains;
   for (size_t i = 0; i < domain_slices.size(); ++i) {
     frag_tile_domains.emplace_back(
@@ -165,6 +165,26 @@ void ReadCellSlabIterFx::create_result_space_tiles(
       result_space_tiles);
 }
 
+void set_result_tile_dim(
+    ResultTile& result_tile,
+    std::string dim,
+    uint64_t dim_idx,
+    std::vector<uint64_t> v) {
+  result_tile.init_coord_tile(dim, dim_idx);
+
+  uint64_t* data =
+      static_cast<uint64_t*>(tdb_malloc(v.size() * sizeof(uint64_t)));
+  for (uint64_t i = 0; i < v.size(); i++) {
+    data[i] = v[i];
+  }
+
+  Tile tile(
+      Datatype::UINT64, sizeof(uint64_t), 0, data, v.size() * sizeof(uint64_t));
+  auto tile_tuple = result_tile.tile_tuple(dim);
+  REQUIRE(tile_tuple != nullptr);
+  std::get<0>(*tile_tuple) = std::move(tile);
+}
+
 /* ********************************* */
 /*                TESTS              */
 /* ********************************* */
@@ -220,13 +240,14 @@ TEST_CASE_METHOD(
   std::vector<NDRange> domain_slices = {ds};
   const auto& tile_coords = subarray.tile_coords();
   std::map<const uint64_t*, ResultSpaceTile<uint64_t>> result_space_tiles;
-  auto dom = array_->array_->array_schema_latest()->domain();
+  auto& dom{array_->array_->array_schema_latest().domain()};
 
-  std::vector<tdb_shared_ptr<FragmentMetadata>> fragments;
-  tdb_shared_ptr<FragmentMetadata> fragment = tdb_make_shared(
-      FragmentMetadata,
+  std::vector<shared_ptr<FragmentMetadata>> fragments;
+  shared_ptr<FragmentMetadata> fragment = make_shared<FragmentMetadata>(
+      HERE(),
       nullptr,
-      array_->array_->array_schema_latest(),
+      nullptr,
+      array_->array_->array_schema_latest_ptr(),
       URI(),
       std::make_pair<uint64_t, uint64_t>(0, 0),
       true);
@@ -235,11 +256,11 @@ TEST_CASE_METHOD(
   create_result_space_tiles(
       fragments,
       dom,
-      dom->domain(),
+      dom.domain(),
       subarray_layout,
       domain_slices,
       tile_coords,
-      &result_space_tiles);
+      result_space_tiles);
 
   // Check iterator
   std::vector<ResultCoords> result_coords;
@@ -292,13 +313,14 @@ TEST_CASE_METHOD(
   std::vector<NDRange> domain_slices = {ds};
   const auto& tile_coords = subarray.tile_coords();
   std::map<const uint64_t*, ResultSpaceTile<uint64_t>> result_space_tiles;
-  auto dom = array_->array_->array_schema_latest()->domain();
+  auto& dom{array_->array_->array_schema_latest().domain()};
 
-  std::vector<tdb_shared_ptr<FragmentMetadata>> fragments;
-  tdb_shared_ptr<FragmentMetadata> fragment = tdb_make_shared(
-      FragmentMetadata,
+  std::vector<shared_ptr<FragmentMetadata>> fragments;
+  shared_ptr<FragmentMetadata> fragment = make_shared<FragmentMetadata>(
+      HERE(),
       nullptr,
-      array_->array_->array_schema_latest(),
+      nullptr,
+      array_->array_->array_schema_latest_ptr(),
       URI(),
       std::make_pair<uint64_t, uint64_t>(0, 0),
       true);
@@ -307,11 +329,11 @@ TEST_CASE_METHOD(
   create_result_space_tiles(
       fragments,
       dom,
-      dom->domain(),
+      dom.domain(),
       subarray_layout,
       domain_slices,
       tile_coords,
-      &result_space_tiles);
+      result_space_tiles);
 
   // Check iterator
   std::vector<ResultCoords> result_coords;
@@ -367,14 +389,15 @@ TEST_CASE_METHOD(
   std::vector<NDRange> domain_slices = {ds1, ds2};
   const auto& tile_coords = subarray.tile_coords();
   std::map<const uint64_t*, ResultSpaceTile<uint64_t>> result_space_tiles;
-  auto dom = array_->array_->array_schema_latest()->domain();
+  auto& dom{array_->array_->array_schema_latest().domain()};
 
-  std::vector<tdb_shared_ptr<FragmentMetadata>> fragments;
+  std::vector<shared_ptr<FragmentMetadata>> fragments;
   for (uint64_t i = 0; i < 2; i++) {
-    tdb_shared_ptr<FragmentMetadata> fragment = tdb_make_shared(
-        FragmentMetadata,
+    shared_ptr<FragmentMetadata> fragment = make_shared<FragmentMetadata>(
+        HERE(),
         nullptr,
-        array_->array_->array_schema_latest(),
+        nullptr,
+        array_->array_->array_schema_latest_ptr(),
         URI(),
         std::make_pair<uint64_t, uint64_t>(0, 0),
         true);
@@ -384,11 +407,11 @@ TEST_CASE_METHOD(
   create_result_space_tiles(
       fragments,
       dom,
-      dom->domain(),
+      dom.domain(),
       subarray_layout,
       domain_slices,
       tile_coords,
-      &result_space_tiles);
+      result_space_tiles);
 
   // Check iterator
   std::vector<ResultCoords> result_coords;
@@ -448,14 +471,15 @@ TEST_CASE_METHOD(
   std::vector<NDRange> domain_slices = {ds};
   const auto& tile_coords = subarray.tile_coords();
   std::map<const uint64_t*, ResultSpaceTile<uint64_t>> result_space_tiles;
-  auto dom = array_->array_->array_schema_latest()->domain();
+  auto& dom{array_->array_->array_schema_latest().domain()};
 
-  std::vector<tdb_shared_ptr<FragmentMetadata>> fragments;
+  std::vector<shared_ptr<FragmentMetadata>> fragments;
   for (uint64_t i = 0; i < 2; i++) {
-    tdb_shared_ptr<FragmentMetadata> fragment = tdb_make_shared(
-        FragmentMetadata,
+    shared_ptr<FragmentMetadata> fragment = make_shared<FragmentMetadata>(
+        HERE(),
         nullptr,
-        array_->array_->array_schema_latest(),
+        nullptr,
+        array_->array_->array_schema_latest_ptr(),
         URI(),
         std::make_pair<uint64_t, uint64_t>(0, 0),
         true);
@@ -465,11 +489,11 @@ TEST_CASE_METHOD(
   create_result_space_tiles(
       fragments,
       dom,
-      dom->domain(),
+      dom.domain(),
       subarray_layout,
       domain_slices,
       tile_coords,
-      &result_space_tiles);
+      result_space_tiles);
 
   // Create result coordinates
   std::vector<ResultCoords> result_coords;
@@ -477,30 +501,9 @@ TEST_CASE_METHOD(
   ResultTile result_tile_3_0(2, 0, array_->array_->array_schema_latest());
   ResultTile result_tile_3_1(2, 1, array_->array_->array_schema_latest());
 
-  result_tile_2_0.init_coord_tile("d", 0);
-  result_tile_3_0.init_coord_tile("d", 0);
-  result_tile_3_1.init_coord_tile("d", 0);
-
-  std::vector<uint64_t> vec_2_0 = {1000, 3, 1000, 5};
-  Buffer buff_2_0(&vec_2_0[0], vec_2_0.size() * sizeof(uint64_t));
-  Tile tile_2_0(Datatype::UINT64, sizeof(uint64_t), 0, &buff_2_0, false);
-  auto tile_tuple = result_tile_2_0.tile_tuple("d");
-  REQUIRE(tile_tuple != nullptr);
-  std::get<0>(*tile_tuple) = tile_2_0;
-
-  std::vector<uint64_t> vec_3_0 = {1000, 1000, 8, 9};
-  Buffer buff_3_0(&vec_3_0[0], vec_3_0.size() * sizeof(uint64_t));
-  Tile tile_3_0(Datatype::UINT64, sizeof(uint64_t), 0, &buff_3_0, false);
-  tile_tuple = result_tile_3_0.tile_tuple("d");
-  REQUIRE(tile_tuple != nullptr);
-  std::get<0>(*tile_tuple) = tile_3_0;
-
-  std::vector<uint64_t> vec_3_1 = {1000, 12, 19, 1000};
-  Buffer buff_3_1(&vec_3_1[0], vec_3_1.size() * sizeof(uint64_t));
-  Tile tile_3_1(Datatype::UINT64, sizeof(uint64_t), 0, &buff_3_1, false);
-  tile_tuple = result_tile_3_1.tile_tuple("d");
-  REQUIRE(tile_tuple != nullptr);
-  std::get<0>(*tile_tuple) = tile_3_1;
+  set_result_tile_dim(result_tile_2_0, "d", 0, {{1000, 3, 1000, 5}});
+  set_result_tile_dim(result_tile_3_0, "d", 0, {{1000, 1000, 8, 9}});
+  set_result_tile_dim(result_tile_3_1, "d", 0, {{1000, 12, 19, 1000}});
 
   result_coords.emplace_back(&result_tile_2_0, 1);
   result_coords.emplace_back(&result_tile_2_0, 3);
@@ -685,13 +688,14 @@ TEST_CASE_METHOD(
   std::vector<NDRange> domain_slices = {ds};
   const auto& tile_coords = subarray.tile_coords();
   std::map<const uint64_t*, ResultSpaceTile<uint64_t>> result_space_tiles;
-  auto dom = array_->array_->array_schema_latest()->domain();
+  auto& dom{array_->array_->array_schema_latest().domain()};
 
-  std::vector<tdb_shared_ptr<FragmentMetadata>> fragments;
-  tdb_shared_ptr<FragmentMetadata> fragment = tdb_make_shared(
-      FragmentMetadata,
+  std::vector<shared_ptr<FragmentMetadata>> fragments;
+  shared_ptr<FragmentMetadata> fragment = make_shared<FragmentMetadata>(
+      HERE(),
       nullptr,
-      array_->array_->array_schema_latest(),
+      nullptr,
+      array_->array_->array_schema_latest_ptr(),
       URI(),
       std::make_pair<uint64_t, uint64_t>(0, 0),
       true);
@@ -700,11 +704,11 @@ TEST_CASE_METHOD(
   create_result_space_tiles(
       fragments,
       dom,
-      dom->domain(),
+      dom.domain(),
       tile_domain_layout,
       domain_slices,
       tile_coords,
-      &result_space_tiles);
+      result_space_tiles);
 
   // Create result coordinates
   std::vector<ResultCoords> result_coords;
@@ -869,13 +873,14 @@ TEST_CASE_METHOD(
   std::vector<NDRange> domain_slices = {ds};
   const auto& tile_coords = subarray.tile_coords();
   std::map<const uint64_t*, ResultSpaceTile<uint64_t>> result_space_tiles;
-  auto dom = array_->array_->array_schema_latest()->domain();
+  auto& dom{array_->array_->array_schema_latest().domain()};
 
-  std::vector<tdb_shared_ptr<FragmentMetadata>> fragments;
-  tdb_shared_ptr<FragmentMetadata> fragment = tdb_make_shared(
-      FragmentMetadata,
+  std::vector<shared_ptr<FragmentMetadata>> fragments;
+  shared_ptr<FragmentMetadata> fragment = make_shared<FragmentMetadata>(
+      HERE(),
       nullptr,
-      array_->array_->array_schema_latest(),
+      nullptr,
+      array_->array_->array_schema_latest_ptr(),
       URI(),
       std::make_pair<uint64_t, uint64_t>(0, 0),
       true);
@@ -884,11 +889,11 @@ TEST_CASE_METHOD(
   create_result_space_tiles(
       fragments,
       dom,
-      dom->domain(),
+      dom.domain(),
       tile_domain_layout,
       domain_slices,
       tile_coords,
-      &result_space_tiles);
+      result_space_tiles);
 
   // Create result coordinates
   std::vector<ResultCoords> result_coords;
@@ -1066,13 +1071,14 @@ TEST_CASE_METHOD(
   std::vector<NDRange> domain_slices = {ds};
   const auto& tile_coords = subarray.tile_coords();
   std::map<const uint64_t*, ResultSpaceTile<uint64_t>> result_space_tiles;
-  auto dom = array_->array_->array_schema_latest()->domain();
+  auto& dom{array_->array_->array_schema_latest().domain()};
 
-  std::vector<tdb_shared_ptr<FragmentMetadata>> fragments;
-  tdb_shared_ptr<FragmentMetadata> fragment = tdb_make_shared(
-      FragmentMetadata,
+  std::vector<shared_ptr<FragmentMetadata>> fragments;
+  shared_ptr<FragmentMetadata> fragment = make_shared<FragmentMetadata>(
+      HERE(),
       nullptr,
-      array_->array_->array_schema_latest(),
+      nullptr,
+      array_->array_->array_schema_latest_ptr(),
       URI(),
       std::make_pair<uint64_t, uint64_t>(0, 0),
       true);
@@ -1081,11 +1087,11 @@ TEST_CASE_METHOD(
   create_result_space_tiles(
       fragments,
       dom,
-      dom->domain(),
+      dom.domain(),
       tile_domain_layout,
       domain_slices,
       tile_coords,
-      &result_space_tiles);
+      result_space_tiles);
 
   // Create result coordinates
   std::vector<ResultCoords> result_coords;
@@ -1309,14 +1315,15 @@ TEST_CASE_METHOD(
   std::vector<NDRange> domain_slices = {ds1, ds2};
   const auto& tile_coords = subarray.tile_coords();
   std::map<const uint64_t*, ResultSpaceTile<uint64_t>> result_space_tiles;
-  auto dom = array_->array_->array_schema_latest()->domain();
+  auto& dom{array_->array_->array_schema_latest().domain()};
 
-  std::vector<tdb_shared_ptr<FragmentMetadata>> fragments;
+  std::vector<shared_ptr<FragmentMetadata>> fragments;
   for (uint64_t i = 0; i < 2; i++) {
-    tdb_shared_ptr<FragmentMetadata> fragment = tdb_make_shared(
-        FragmentMetadata,
+    shared_ptr<FragmentMetadata> fragment = make_shared<FragmentMetadata>(
+        HERE(),
         nullptr,
-        array_->array_->array_schema_latest(),
+        nullptr,
+        array_->array_->array_schema_latest_ptr(),
         URI(),
         std::make_pair<uint64_t, uint64_t>(0, 0),
         true);
@@ -1326,49 +1333,21 @@ TEST_CASE_METHOD(
   create_result_space_tiles(
       fragments,
       dom,
-      dom->domain(),
+      dom.domain(),
       tile_domain_layout,
       domain_slices,
       tile_coords,
-      &result_space_tiles);
+      result_space_tiles);
 
   // Create result coordinates
   std::vector<ResultCoords> result_coords;
   ResultTile result_tile_3_0(2, 0, array_->array_->array_schema_latest());
   ResultTile result_tile_3_1(2, 1, array_->array_->array_schema_latest());
 
-  result_tile_3_0.init_coord_tile("d1", 0);
-  result_tile_3_0.init_coord_tile("d2", 1);
-  result_tile_3_1.init_coord_tile("d1", 0);
-  result_tile_3_1.init_coord_tile("d2", 1);
-
-  std::vector<uint64_t> vec_3_0_d1 = {1000, 3, 1000, 1000};
-  Buffer buff_3_0_d1(&vec_3_0_d1[0], vec_3_0_d1.size() * sizeof(uint64_t));
-  Tile tile_3_0_d1(Datatype::UINT64, sizeof(uint64_t), 0, &buff_3_0_d1, false);
-  auto tile_tuple = result_tile_3_0.tile_tuple("d1");
-  REQUIRE(tile_tuple != nullptr);
-  std::get<0>(*tile_tuple) = tile_3_0_d1;
-
-  std::vector<uint64_t> vec_3_0_d2 = {1000, 3, 1000, 1000};
-  Buffer buff_3_0_d2(&vec_3_0_d2[0], vec_3_0_d2.size() * sizeof(uint64_t));
-  Tile tile_3_0_d2(Datatype::UINT64, sizeof(uint64_t), 0, &buff_3_0_d2, false);
-  tile_tuple = result_tile_3_0.tile_tuple("d2");
-  REQUIRE(tile_tuple != nullptr);
-  std::get<0>(*tile_tuple) = tile_3_0_d2;
-
-  std::vector<uint64_t> vec_3_1_d1 = {5, 1000, 5, 1000};
-  Buffer buff_3_1_d1(&vec_3_1_d1[0], vec_3_1_d1.size() * sizeof(uint64_t));
-  Tile tile_3_1_d1(Datatype::UINT64, sizeof(uint64_t), 0, &buff_3_1_d1, false);
-  tile_tuple = result_tile_3_1.tile_tuple("d1");
-  REQUIRE(tile_tuple != nullptr);
-  std::get<0>(*tile_tuple) = tile_3_1_d1;
-
-  std::vector<uint64_t> vec_3_1_d2 = {5, 1000, 6, 1000};
-  Buffer buff_3_1_d2(&vec_3_1_d2[0], vec_3_1_d2.size() * sizeof(uint64_t));
-  Tile tile_3_1_d2(Datatype::UINT64, sizeof(uint64_t), 0, &buff_3_1_d2, false);
-  tile_tuple = result_tile_3_1.tile_tuple("d2");
-  REQUIRE(tile_tuple != nullptr);
-  std::get<0>(*tile_tuple) = tile_3_1_d2;
+  set_result_tile_dim(result_tile_3_0, "d1", 0, {{1000, 3, 1000, 1000}});
+  set_result_tile_dim(result_tile_3_0, "d2", 1, {{1000, 3, 1000, 1000}});
+  set_result_tile_dim(result_tile_3_1, "d1", 0, {{5, 1000, 5, 1000}});
+  set_result_tile_dim(result_tile_3_1, "d2", 1, {{5, 1000, 6, 1000}});
 
   result_coords.emplace_back(&result_tile_3_0, 1);
   result_coords.emplace_back(&result_tile_3_1, 0);
