@@ -50,6 +50,7 @@
 #include "tiledb/sm/c_api/tiledb_experimental.h"
 #include "tiledb/sm/c_api/tiledb_serialization.h"
 #include "tiledb/sm/enums/serialization_type.h"
+#include "tiledb/sm/misc/constants.h"
 #include "tiledb/sm/misc/utils.h"
 
 using namespace tiledb::test;
@@ -1188,14 +1189,14 @@ TEST_CASE_METHOD(
   const void* extent = NULL;
   rc = tiledb_dimension_get_tile_extent(ctx_, r_d1, &extent);
   REQUIRE(rc == TILEDB_OK);
-  REQUIRE(extent != NULL);
+  REQUIRE(extent != nullptr);
   CHECK(*(const int32_t*)extent == 100);
   tiledb_dimension_t* r_d2;
   rc = tiledb_domain_get_dimension_from_index(ctx_, r_domain, 1, &r_d2);
   REQUIRE(rc == TILEDB_OK);
   rc = tiledb_dimension_get_tile_extent(ctx_, r_d2, &extent);
   REQUIRE(rc == TILEDB_OK);
-  REQUIRE(extent != NULL);
+  REQUIRE(extent != nullptr);
   CHECK(*(const float*)extent == d2_dom[1] - d2_dom[0]);
 
   // Clean up
@@ -1435,7 +1436,7 @@ TEST_CASE_METHOD(
 
   // Corrupt the array schema
   std::string schema_path =
-      array_name + "/" + tiledb::sm::constants::array_schema_folder_name;
+      array_name + "/" + tiledb::sm::constants::array_schema_dir_name;
   std::string to_write = "garbage";
   tiledb_vfs_fh_t* fh;
   schema_file_struct data_struct = {ctx_, vfs_, ""};
@@ -1615,6 +1616,76 @@ TEST_CASE_METHOD(
   remove_temp_dir(local_fs.file_prefix() + local_fs.temp_dir());
 }
 
+TEST_CASE_METHOD(
+    ArraySchemaFx,
+    "C API: Test array schema setter/getter for version",
+    "[capi][array-schema][version]") {
+  SECTION("- No serialization") {
+    serialize_array_schema_ = false;
+  }
+  SECTION("- Serialization") {
+    serialize_array_schema_ = true;
+  }
+
+  // Create and allocate array schema
+  tiledb_array_schema_t* array_schema;
+  int rc = tiledb_array_schema_alloc(ctx_, TILEDB_SPARSE, &array_schema);
+  REQUIRE(rc == TILEDB_OK);
+
+  // Create dimension
+  tiledb_dimension_t* d;
+  rc = tiledb_dimension_alloc(
+      ctx_, "d", TILEDB_INT64, &DIM_DOMAIN[0], &TILE_EXTENTS[0], &d);
+  REQUIRE(rc == TILEDB_OK);
+
+  // Set domain
+  tiledb_domain_t* domain;
+  rc = tiledb_domain_alloc(ctx_, &domain);
+  REQUIRE(rc == TILEDB_OK);
+  rc = tiledb_domain_add_dimension(ctx_, domain, d);
+  REQUIRE(rc == TILEDB_OK);
+  rc = tiledb_array_schema_set_domain(ctx_, array_schema, domain);
+  REQUIRE(rc == TILEDB_OK);
+
+  // Set attribute
+  tiledb_attribute_t* a;
+  rc = tiledb_attribute_alloc(ctx_, "a", ATTR_TYPE, &a);
+  REQUIRE(rc == TILEDB_OK);
+  rc = tiledb_array_schema_add_attribute(ctx_, array_schema, a);
+  REQUIRE(rc == TILEDB_OK);
+
+  // Instantiate local class
+  SupportedFsLocal local_fs;
+
+  // Create array
+  std::string array_name =
+      local_fs.file_prefix() + local_fs.temp_dir() + "duplicates";
+  create_temp_dir(local_fs.file_prefix() + local_fs.temp_dir());
+  rc = array_create_wrapper(array_name, array_schema);
+  REQUIRE(rc == TILEDB_OK);
+
+  // Clean up
+  tiledb_attribute_free(&a);
+  tiledb_dimension_free(&d);
+  tiledb_domain_free(&domain);
+  tiledb_array_schema_free(&array_schema);
+
+  // Load array schema
+  rc = array_schema_load_wrapper(array_name.c_str(), &array_schema);
+  REQUIRE(rc == TILEDB_OK);
+
+  // Get version.
+  uint32_t version_r = 0;
+  rc = tiledb_array_schema_get_version(ctx_, array_schema, &version_r);
+  CHECK(rc == TILEDB_OK);
+  CHECK(version_r == tiledb::sm::constants::format_version);
+
+  // Clean up
+  tiledb_array_schema_free(&array_schema);
+  delete_array(array_name);
+  remove_temp_dir(local_fs.file_prefix() + local_fs.temp_dir());
+}
+
 TEST_CASE_METHOD(
     ArraySchemaFx,
     "C API: Test array schema setter/getter for dimension filters and cell val "
@@ -2039,6 +2110,12 @@ TEST_CASE_METHOD(
       ctx_, array_schema_evolution, "a3");
   REQUIRE(rc == TILEDB_OK);
 
+  // Set timestamp to avoid race condition
+  uint64_t now = tiledb_timestamp_now_ms();
+  now = now + 1;
+  rc = tiledb_array_schema_evolution_set_timestamp_range(
+      ctx_, array_schema_evolution, now, now);
+
   // Evolve schema
   rc = tiledb_array_evolve_wrapper(
       ctx_, array_name.c_str(), array_schema_evolution);
@@ -2175,6 +2252,12 @@ TEST_CASE_METHOD(
       ctx_, array_schema_evolution, "a1");
   REQUIRE(rc == TILEDB_OK);
 
+  // Set timestamp to avoid race condition
+  uint64_t now = tiledb_timestamp_now_ms();
+  now = now + 1;
+  rc = tiledb_array_schema_evolution_set_timestamp_range(
+      ctx_, array_schema_evolution, now, now);
+
   // Evolve schema
   rc = tiledb_array_evolve_wrapper(
       ctx_, array_name.c_str(), array_schema_evolution);
@@ -2241,7 +2324,7 @@ TEST_CASE_METHOD(
   std::string array_uri(arrays_dir + "/non_split_coords_v1_4_0");
   // Remove any failed tests
   remove_temp_dir(
-      array_uri + "/" + tiledb::sm::constants::array_schema_folder_name);
+      array_uri + "/" + tiledb::sm::constants::array_schema_dir_name);
 
   // Create an array schema evolution
   tiledb_array_schema_evolution_t* array_schema_evolution;
@@ -2261,6 +2344,12 @@ TEST_CASE_METHOD(
       ctx_, array_schema_evolution, "a");
   REQUIRE(rc == TILEDB_OK);
 
+  // Set timestamp to avoid race condition
+  uint64_t now = tiledb_timestamp_now_ms();
+  now = now + 1;
+  rc = tiledb_array_schema_evolution_set_timestamp_range(
+      ctx_, array_schema_evolution, now, now);
+
   // Evolve schema
   rc = tiledb_array_evolve_wrapper(
       ctx_, array_uri.c_str(), array_schema_evolution);
@@ -2304,5 +2393,5 @@ TEST_CASE_METHOD(
   tiledb_array_free(&array);
   remove_temp_dir(local_fs.file_prefix() + local_fs.temp_dir());
   remove_temp_dir(
-      array_uri + "/" + tiledb::sm::constants::array_schema_folder_name);
+      array_uri + "/" + tiledb::sm::constants::array_schema_dir_name);
 }
