@@ -30,8 +30,10 @@
  * This file implements class Array.
  */
 
-#include "tiledb/sm/array/array.h"
+#include "tiledb/common/common.h"
+
 #include "tiledb/common/logger.h"
+#include "tiledb/sm/array/array.h"
 #include "tiledb/sm/array_schema/array_schema.h"
 #include "tiledb/sm/array_schema/array_schema_evolution.h"
 #include "tiledb/sm/array_schema/attribute.h"
@@ -44,6 +46,7 @@
 #include "tiledb/sm/enums/serialization_type.h"
 #include "tiledb/sm/fragment/fragment_metadata.h"
 #include "tiledb/sm/global_state/unit_test_config.h"
+#include "tiledb/sm/misc/tdb_time.h"
 #include "tiledb/sm/misc/utils.h"
 #include "tiledb/sm/rest/rest_client.h"
 #include "tiledb/sm/storage_manager/storage_manager.h"
@@ -64,8 +67,9 @@ namespace sm {
 Array::Array(const URI& array_uri, StorageManager* storage_manager)
     : array_schema_latest_(nullptr)
     , array_uri_(array_uri)
+    , array_dir_()
     , array_uri_serialized_(array_uri)
-    , encryption_key_(tdb_make_shared(EncryptionKey))
+    , encryption_key_(make_shared<EncryptionKey>(HERE()))
     , is_open_(false)
     , timestamp_start_(0)
     , timestamp_end_(UINT64_MAX)
@@ -74,11 +78,13 @@ Array::Array(const URI& array_uri, StorageManager* storage_manager)
     , config_(storage_manager_->config())
     , remote_(array_uri.is_tiledb())
     , metadata_loaded_(false)
-    , non_empty_domain_computed_(false){};
+    , non_empty_domain_computed_(false) {
+}
 
 Array::Array(const Array& rhs)
     : array_schema_latest_(rhs.array_schema_latest_)
     , array_uri_(rhs.array_uri_)
+    , array_dir_(rhs.array_dir_)
     , array_uri_serialized_(rhs.array_uri_serialized_)
     , encryption_key_(rhs.encryption_key_)
     , fragment_metadata_(rhs.fragment_metadata_)
@@ -101,18 +107,28 @@ Array::Array(const Array& rhs)
 /*                API                */
 /* ********************************* */
 
-ArraySchema* Array::array_schema_latest() const {
-  std::unique_lock<std::mutex> lck(mtx_);
+void Array::set_array_schema_latest(
+    const shared_ptr<ArraySchema>& array_schema) {
+  array_schema_latest_ = array_schema;
+}
+
+const ArraySchema& Array::array_schema_latest() const {
+  return *(array_schema_latest_.get());
+}
+
+shared_ptr<const ArraySchema> Array::array_schema_latest_ptr() const {
   return array_schema_latest_;
 }
 
 const URI& Array::array_uri() const {
-  std::unique_lock<std::mutex> lck(mtx_);
   return array_uri_;
 }
 
+const ArrayDirectory& Array::array_directory() const {
+  return array_dir_;
+}
+
 const URI& Array::array_uri_serialized() const {
-  std::unique_lock<std::mutex> lck(mtx_);
   return array_uri_serialized_;
 }
 
@@ -125,15 +141,21 @@ Status Array::open_without_fragments(
     EncryptionType encryption_type,
     const void* encryption_key,
     uint32_t key_length) {
-  std::unique_lock<std::mutex> lck(mtx_);
-
+  // Checks
   if (is_open_)
-    return LOG_STATUS(Status::ArrayError(
+    return LOG_STATUS(Status_ArrayError(
         "Cannot open array without fragments; Array already open"));
-
   if (remote_ && encryption_type != EncryptionType::NO_ENCRYPTION)
-    return LOG_STATUS(Status::ArrayError(
-        "Cannot open array; encrypted remote arrays are not supported."));
+    return LOG_STATUS(
+        Status_ArrayError("Cannot open array without fragments; encrypted "
+                          "remote arrays are not supported."));
+  if (!remote_) {
+    bool is_array = false;
+    RETURN_NOT_OK(storage_manager_->is_array(array_uri_, &is_array));
+    if (!is_array)
+      return LOG_STATUS(Status_ArrayError(
+          "Cannot open array without fragments; Array does not exist"));
+  }
 
   // Copy the key bytes.
   RETURN_NOT_OK(
@@ -146,16 +168,28 @@ Status Array::open_without_fragments(
   if (remote_) {
     auto rest_client = storage_manager_->rest_client();
     if (rest_client == nullptr)
-      return LOG_STATUS(Status::ArrayError(
+      return LOG_STATUS(Status_ArrayError(
           "Cannot open array; remote array with no REST client."));
-    RETURN_NOT_OK(rest_client->get_array_schema_from_rest(
-        array_uri_, &array_schema_latest_));
+    auto&& [st, array_schema_latest] =
+        rest_client->get_array_schema_from_rest(array_uri_);
+    RETURN_NOT_OK(st);
+    array_schema_latest_ = array_schema_latest.value();
   } else {
+    try {
+      array_dir_ = ArrayDirectory(
+          storage_manager_->vfs(),
+          storage_manager_->compute_tp(),
+          array_uri_,
+          0,
+          UINT64_MAX,
+          ArrayDirectoryMode::SCHEMA_ONLY);
+    } catch (const std::logic_error& le) {
+      return LOG_STATUS(Status_ArrayDirectoryError(le.what()));
+    }
+
     auto&& [st, array_schema, array_schemas] =
-        storage_manager_->array_open_for_reads_without_fragments(
-            array_uri_, *encryption_key_);
-    if (!st.ok())
-      return st;
+        storage_manager_->array_open_for_reads_without_fragments(this);
+    RETURN_NOT_OK(st);
 
     array_schema_latest_ = array_schema.value();
     array_schemas_all_ = array_schemas.value();
@@ -169,11 +203,13 @@ Status Array::open_without_fragments(
 
 Status Array::load_fragments(
     const std::vector<TimestampedURI>& fragments_to_load) {
-  return storage_manager_->array_load_fragments(
-      array_uri_,
-      *encryption_key_.get(),
-      &fragment_metadata_,
-      fragments_to_load);
+  auto&& [st, fragment_metadata] =
+      storage_manager_->array_load_fragments(this, fragments_to_load);
+  RETURN_NOT_OK(st);
+
+  fragment_metadata_ = std::move(fragment_metadata.value());
+
+  return Status::Ok();
 }
 
 Status Array::open(
@@ -197,12 +233,19 @@ Status Array::open(
     EncryptionType encryption_type,
     const void* encryption_key,
     uint32_t key_length) {
-  std::unique_lock<std::mutex> lck(mtx_);
-
+  // Checks
   if (is_open_)
     return LOG_STATUS(
-        Status::ArrayError("Cannot open array; Array already open"));
+        Status_ArrayError("Cannot open array; Array already open"));
+  if (!remote_) {
+    bool is_array = false;
+    RETURN_NOT_OK(storage_manager_->is_array(array_uri_, &is_array));
+    if (!is_array)
+      return LOG_STATUS(
+          Status_ArrayError("Cannot open array; Array does not exist"));
+  }
 
+  // Get encryption key from config
   std::string encryption_key_from_cfg;
   if (!encryption_key) {
     bool found = false;
@@ -236,7 +279,7 @@ Status Array::open(
   }
 
   if (remote_ && encryption_type != EncryptionType::NO_ENCRYPTION)
-    return LOG_STATUS(Status::ArrayError(
+    return LOG_STATUS(Status_ArrayError(
         "Cannot open array; encrypted remote arrays are not supported."));
 
   // Copy the key bytes.
@@ -262,30 +305,48 @@ Status Array::open(
   if (remote_) {
     auto rest_client = storage_manager_->rest_client();
     if (rest_client == nullptr)
-      return LOG_STATUS(Status::ArrayError(
+      return LOG_STATUS(Status_ArrayError(
           "Cannot open array; remote array with no REST client."));
-    RETURN_NOT_OK(rest_client->get_array_schema_from_rest(
-        array_uri_, &array_schema_latest_));
+    auto&& [st, array_schema_latest] =
+        rest_client->get_array_schema_from_rest(array_uri_);
+    RETURN_NOT_OK(st);
+    array_schema_latest_ = array_schema_latest.value();
   } else if (query_type == QueryType::READ) {
-    auto&& [st, array_schema, array_schemas, fragment_metadata] =
-        storage_manager_->array_open_for_reads(
-            array_uri_,
-            *encryption_key_,
-            timestamp_start_,
-            timestamp_end_opened_at_);
-    if (!st.ok())
-      return st;
+    try {
+      array_dir_ = ArrayDirectory(
+          storage_manager_->vfs(),
+          storage_manager_->compute_tp(),
+          array_uri_,
+          timestamp_start_,
+          timestamp_end_opened_at_);
+    } catch (const std::logic_error& le) {
+      return LOG_STATUS(Status_ArrayDirectoryError(le.what()));
+    }
+
+    auto&& [st, array_schema_latest, array_schemas, fragment_metadata] =
+        storage_manager_->array_open_for_reads(this);
+    RETURN_NOT_OK(st);
     // Set schemas
-    array_schema_latest_ = array_schema.value();
+    array_schema_latest_ = array_schema_latest.value();
     array_schemas_all_ = array_schemas.value();
     fragment_metadata_ = fragment_metadata.value();
   } else {
-    auto&& [st, array_schema, array_schemas] =
-        storage_manager_->array_open_for_writes(array_uri_, *encryption_key_);
-    if (!st.ok())
-      return st;
+    try {
+      array_dir_ = ArrayDirectory(
+          storage_manager_->vfs(),
+          storage_manager_->compute_tp(),
+          array_uri_,
+          timestamp_start_,
+          timestamp_end_opened_at_);
+    } catch (const std::logic_error& le) {
+      return LOG_STATUS(Status_ArrayDirectoryError(le.what()));
+    }
+
+    auto&& [st, array_schema_latest, array_schemas] =
+        storage_manager_->array_open_for_writes(this);
+    RETURN_NOT_OK(st);
     // Set schemas
-    array_schema_latest_ = array_schema.value();
+    array_schema_latest_ = array_schema_latest.value();
     array_schemas_all_ = array_schemas.value();
 
     metadata_.reset(timestamp_end_opened_at_);
@@ -298,12 +359,13 @@ Status Array::open(
 }
 
 Status Array::close() {
-  std::unique_lock<std::mutex> lck(mtx_);
-
-  if (!is_open_)
+  // Check if array is open
+  if (!is_open_) {
+    // If array is not open treat this as a no-op
+    // This keeps existing behavior from TileDB 2.6 and older
     return Status::Ok();
+  }
 
-  is_open_ = false;
   non_empty_domain_.clear();
   non_empty_domain_computed_ = false;
   clear_last_max_buffer_sizes();
@@ -318,38 +380,36 @@ Status Array::close() {
       metadata_loaded_ = true;
       auto rest_client = storage_manager_->rest_client();
       if (rest_client == nullptr)
-        return LOG_STATUS(Status::ArrayError(
+        return LOG_STATUS(Status_ArrayError(
             "Error closing array; remote array with no REST client."));
       RETURN_NOT_OK(rest_client->post_array_metadata_to_rest(
           array_uri_, timestamp_start_, timestamp_end_opened_at_, this));
     }
 
     // Storage manager does not own the array schema for remote arrays.
-    tdb_delete(array_schema_latest_);
-    array_schema_latest_ = nullptr;
+    array_schema_latest_.reset();
   } else {
-    array_schema_latest_ = nullptr;
+    array_schema_latest_.reset();
     if (query_type_ == QueryType::READ) {
-      RETURN_NOT_OK(storage_manager_->array_close_for_reads(array_uri_));
+      RETURN_NOT_OK(storage_manager_->array_close_for_reads(this));
     } else {
-      RETURN_NOT_OK(storage_manager_->array_close_for_writes(
-          array_uri_, *encryption_key_, &metadata_));
+      RETURN_NOT_OK(storage_manager_->array_close_for_writes(this));
     }
   }
 
+  array_schemas_all_.clear();
   metadata_.clear();
   metadata_loaded_ = false;
+  is_open_ = false;
 
   return Status::Ok();
 }
 
 bool Array::is_empty() const {
-  std::unique_lock<std::mutex> lck(mtx_);
   return fragment_metadata_.empty();
 }
 
 bool Array::is_open() const {
-  std::unique_lock<std::mutex> lck(mtx_);
   return is_open_;
 }
 
@@ -357,31 +417,26 @@ bool Array::is_remote() const {
   return remote_;
 }
 
-std::vector<tdb_shared_ptr<FragmentMetadata>> Array::fragment_metadata() const {
-  std::unique_lock<std::mutex> lck(mtx_);
+std::vector<shared_ptr<FragmentMetadata>> Array::fragment_metadata() const {
   return fragment_metadata_;
 }
 
-Status Array::get_array_schema(ArraySchema** array_schema) const {
-  std::unique_lock<std::mutex> lck(mtx_);
-
+tuple<Status, optional<shared_ptr<ArraySchema>>> Array::get_array_schema()
+    const {
   // Error if the array is not open
   if (!is_open_)
-    return LOG_STATUS(
-        Status::ArrayError("Cannot get array schema; Array is not open"));
+    return {LOG_STATUS(Status_ArrayError(
+                "Cannot get array schema; Array is not open")),
+            nullopt};
 
-  *array_schema = array_schema_latest_;
-
-  return Status::Ok();
+  return {Status::Ok(), array_schema_latest_};
 }
 
 Status Array::get_query_type(QueryType* query_type) const {
-  std::unique_lock<std::mutex> lck(mtx_);
-
   // Error if the array is not open
   if (!is_open_)
     return LOG_STATUS(
-        Status::ArrayError("Cannot get query_type; Array is not open"));
+        Status_ArrayError("Cannot get query_type; Array is not open"));
 
   *query_type = query_type_;
 
@@ -390,32 +445,31 @@ Status Array::get_query_type(QueryType* query_type) const {
 
 Status Array::get_max_buffer_size(
     const char* name, const void* subarray, uint64_t* buffer_size) {
-  std::unique_lock<std::mutex> lck(mtx_);
   // Check if array is open
   if (!is_open_)
     return LOG_STATUS(
-        Status::ArrayError("Cannot get max buffer size; Array is not open"));
+        Status_ArrayError("Cannot get max buffer size; Array is not open"));
 
   // Error if the array was not opened in read mode
   if (query_type_ != QueryType::READ)
     return LOG_STATUS(
-        Status::ArrayError("Cannot get max buffer size; "
-                           "Array was not opened in read mode"));
+        Status_ArrayError("Cannot get max buffer size; "
+                          "Array was not opened in read mode"));
 
   // Check if name is null
   if (name == nullptr)
-    return LOG_STATUS(Status::ArrayError(
+    return LOG_STATUS(Status_ArrayError(
         "Cannot get max buffer size; Attribute/Dimension name is null"));
 
   // Not applicable to heterogeneous domains
-  if (!array_schema_latest_->domain()->all_dims_same_type())
+  if (!array_schema_latest_->domain().all_dims_same_type())
     return LOG_STATUS(
-        Status::ArrayError("Cannot get max buffer size; Function not "
-                           "applicable to heterogeneous domains"));
+        Status_ArrayError("Cannot get max buffer size; Function not "
+                          "applicable to heterogeneous domains"));
 
   // Not applicable to variable-sized dimensions
-  if (!array_schema_latest_->domain()->all_dims_fixed())
-    return LOG_STATUS(Status::ArrayError(
+  if (!array_schema_latest_->domain().all_dims_fixed())
+    return LOG_STATUS(Status_ArrayError(
         "Cannot get max buffer size; Function not "
         "applicable to domains with variable-sized dimensions"));
 
@@ -425,13 +479,13 @@ Status Array::get_max_buffer_size(
 
   // Check if attribute/dimension exists
   if (name != constants::coords && !is_dim && !is_attr)
-    return LOG_STATUS(Status::ArrayError(
+    return LOG_STATUS(Status_ArrayError(
         std::string("Cannot get max buffer size; Attribute/Dimension '") +
         name + "' does not exist"));
 
   // Check if attribute/dimension is fixed sized
   if (array_schema_latest_->var_size(name))
-    return LOG_STATUS(Status::ArrayError(
+    return LOG_STATUS(Status_ArrayError(
         std::string("Cannot get max buffer size; Attribute/Dimension '") +
         name + "' is var-sized"));
 
@@ -450,33 +504,31 @@ Status Array::get_max_buffer_size(
     const void* subarray,
     uint64_t* buffer_off_size,
     uint64_t* buffer_val_size) {
-  std::unique_lock<std::mutex> lck(mtx_);
-
   // Check if array is open
   if (!is_open_)
     return LOG_STATUS(
-        Status::ArrayError("Cannot get max buffer size; Array is not open"));
+        Status_ArrayError("Cannot get max buffer size; Array is not open"));
 
   // Error if the array was not opened in read mode
   if (query_type_ != QueryType::READ)
     return LOG_STATUS(
-        Status::ArrayError("Cannot get max buffer size; "
-                           "Array was not opened in read mode"));
+        Status_ArrayError("Cannot get max buffer size; "
+                          "Array was not opened in read mode"));
 
   // Check if name is null
   if (name == nullptr)
-    return LOG_STATUS(Status::ArrayError(
+    return LOG_STATUS(Status_ArrayError(
         "Cannot get max buffer size; Attribute/Dimension name is null"));
 
   // Not applicable to heterogeneous domains
-  if (!array_schema_latest_->domain()->all_dims_same_type())
+  if (!array_schema_latest_->domain().all_dims_same_type())
     return LOG_STATUS(
-        Status::ArrayError("Cannot get max buffer size; Function not "
-                           "applicable to heterogeneous domains"));
+        Status_ArrayError("Cannot get max buffer size; Function not "
+                          "applicable to heterogeneous domains"));
 
   // Not applicable to variable-sized dimensions
-  if (!array_schema_latest_->domain()->all_dims_fixed())
-    return LOG_STATUS(Status::ArrayError(
+  if (!array_schema_latest_->domain().all_dims_fixed())
+    return LOG_STATUS(Status_ArrayError(
         "Cannot get max buffer size; Function not "
         "applicable to domains with variable-sized dimensions"));
 
@@ -485,13 +537,13 @@ Status Array::get_max_buffer_size(
   // Check if attribute/dimension exists
   auto it = last_max_buffer_sizes_.find(name);
   if (it == last_max_buffer_sizes_.end())
-    return LOG_STATUS(Status::ArrayError(
+    return LOG_STATUS(Status_ArrayError(
         std::string("Cannot get max buffer size; Attribute/Dimension '") +
         name + "' does not exist"));
 
   // Check if attribute/dimension is var-sized
   if (!array_schema_latest_->var_size(name))
-    return LOG_STATUS(Status::ArrayError(
+    return LOG_STATUS(Status_ArrayError(
         std::string("Cannot get max buffer size; Attribute/Dimension '") +
         name + "' is fixed-sized"));
 
@@ -503,7 +555,6 @@ Status Array::get_max_buffer_size(
 }
 
 const EncryptionKey& Array::get_encryption_key() const {
-  std::unique_lock<std::mutex> lck(mtx_);
   return *encryption_key_;
 }
 
@@ -519,16 +570,14 @@ Status Array::reopen() {
 }
 
 Status Array::reopen(uint64_t timestamp_start, uint64_t timestamp_end) {
-  std::unique_lock<std::mutex> lck(mtx_);
-
   if (!is_open_)
     return LOG_STATUS(
-        Status::ArrayError("Cannot reopen array; Array is not open"));
+        Status_ArrayError("Cannot reopen array; Array is not open"));
 
   if (query_type_ != QueryType::READ)
     return LOG_STATUS(
-        Status::ArrayError("Cannot reopen array; Array was "
-                           "not opened in read mode"));
+        Status_ArrayError("Cannot reopen array; Array was "
+                          "not opened in read mode"));
 
   clear_last_max_buffer_sizes();
 
@@ -552,17 +601,22 @@ Status Array::reopen(uint64_t timestamp_start, uint64_t timestamp_end) {
         encryption_key_->key().size());
   }
 
-  auto&& [st, array_schema, array_schemas, fragment_metadata] =
-      storage_manager_->array_reopen(
-          array_uri_,
-          *encryption_key_,
-          timestamp_start_,
-          timestamp_end_opened_at_);
+  try {
+    array_dir_ = ArrayDirectory(
+        storage_manager_->vfs(),
+        storage_manager_->compute_tp(),
+        array_uri_,
+        timestamp_start_,
+        timestamp_end_opened_at_);
+  } catch (const std::logic_error& le) {
+    return LOG_STATUS(Status_ArrayDirectoryError(le.what()));
+  }
 
-  if (!st.ok())
-    return st;
+  auto&& [st, array_schema_latest, array_schemas, fragment_metadata] =
+      storage_manager_->array_reopen(this);
+  RETURN_NOT_OK(st);
 
-  array_schema_latest_ = array_schema.value();
+  array_schema_latest_ = array_schema_latest.value();
   array_schemas_all_ = array_schemas.value();
   fragment_metadata_ = fragment_metadata.value();
 
@@ -570,29 +624,24 @@ Status Array::reopen(uint64_t timestamp_start, uint64_t timestamp_end) {
 }
 
 Status Array::set_timestamp_start(const uint64_t timestamp_start) {
-  std::unique_lock<std::mutex> lck(mtx_);
   timestamp_start_ = timestamp_start;
   return Status::Ok();
 }
 
 uint64_t Array::timestamp_start() const {
-  std::unique_lock<std::mutex> lck(mtx_);
   return timestamp_start_;
 }
 
 Status Array::set_timestamp_end(const uint64_t timestamp_end) {
-  std::unique_lock<std::mutex> lck(mtx_);
   timestamp_end_ = timestamp_end;
   return Status::Ok();
 }
 
 uint64_t Array::timestamp_end() const {
-  std::unique_lock<std::mutex> lck(mtx_);
   return timestamp_end_;
 }
 
 uint64_t Array::timestamp_end_opened_at() const {
-  std::unique_lock<std::mutex> lck(mtx_);
   return timestamp_end_opened_at_;
 }
 
@@ -605,14 +654,7 @@ Config Array::config() const {
   return config_;
 }
 
-Status Array::set_uri(const std::string& uri) {
-  std::unique_lock<std::mutex> lck(mtx_);
-  array_uri_ = URI(uri);
-  return Status::Ok();
-}
-
 Status Array::set_uri_serialized(const std::string& uri) {
-  std::unique_lock<std::mutex> lck(mtx_);
   array_uri_serialized_ = URI(uri);
   return Status::Ok();
 }
@@ -621,18 +663,18 @@ Status Array::delete_metadata(const char* key) {
   // Check if array is open
   if (!is_open_)
     return LOG_STATUS(
-        Status::ArrayError("Cannot delete metadata. Array is not open"));
+        Status_ArrayError("Cannot delete metadata. Array is not open"));
 
   // Check mode
   if (query_type_ != QueryType::WRITE)
     return LOG_STATUS(
-        Status::ArrayError("Cannot delete metadata. Array was "
-                           "not opened in write mode"));
+        Status_ArrayError("Cannot delete metadata. Array was "
+                          "not opened in write mode"));
 
   // Check if key is null
   if (key == nullptr)
     return LOG_STATUS(
-        Status::ArrayError("Cannot delete metadata. Key cannot be null"));
+        Status_ArrayError("Cannot delete metadata. Key cannot be null"));
 
   RETURN_NOT_OK(metadata_.del(key));
 
@@ -647,23 +689,23 @@ Status Array::put_metadata(
   // Check if array is open
   if (!is_open_)
     return LOG_STATUS(
-        Status::ArrayError("Cannot put metadata; Array is not open"));
+        Status_ArrayError("Cannot put metadata; Array is not open"));
 
   // Check mode
   if (query_type_ != QueryType::WRITE)
     return LOG_STATUS(
-        Status::ArrayError("Cannot put metadata; Array was "
-                           "not opened in write mode"));
+        Status_ArrayError("Cannot put metadata; Array was "
+                          "not opened in write mode"));
 
   // Check if key is null
   if (key == nullptr)
     return LOG_STATUS(
-        Status::ArrayError("Cannot put metadata; Key cannot be null"));
+        Status_ArrayError("Cannot put metadata; Key cannot be null"));
 
   // Check if value type is ANY
   if (value_type == Datatype::ANY)
     return LOG_STATUS(
-        Status::ArrayError("Cannot put metadata; Value type cannot be ANY"));
+        Status_ArrayError("Cannot put metadata; Value type cannot be ANY"));
 
   RETURN_NOT_OK(metadata_.put(key, value_type, value_num, value));
 
@@ -678,18 +720,18 @@ Status Array::get_metadata(
   // Check if array is open
   if (!is_open_)
     return LOG_STATUS(
-        Status::ArrayError("Cannot get metadata; Array is not open"));
+        Status_ArrayError("Cannot get metadata; Array is not open"));
 
   // Check mode
   if (query_type_ != QueryType::READ)
     return LOG_STATUS(
-        Status::ArrayError("Cannot get metadata; Array was "
-                           "not opened in read mode"));
+        Status_ArrayError("Cannot get metadata; Array was "
+                          "not opened in read mode"));
 
   // Check if key is null
   if (key == nullptr)
     return LOG_STATUS(
-        Status::ArrayError("Cannot get metadata; Key cannot be null"));
+        Status_ArrayError("Cannot get metadata; Key cannot be null"));
 
   // Load array metadata, if not loaded yet
   if (!metadata_loaded_)
@@ -710,13 +752,13 @@ Status Array::get_metadata(
   // Check if array is open
   if (!is_open_)
     return LOG_STATUS(
-        Status::ArrayError("Cannot get metadata; Array is not open"));
+        Status_ArrayError("Cannot get metadata; Array is not open"));
 
   // Check mode
   if (query_type_ != QueryType::READ)
     return LOG_STATUS(
-        Status::ArrayError("Cannot get metadata; Array was "
-                           "not opened in read mode"));
+        Status_ArrayError("Cannot get metadata; Array was "
+                          "not opened in read mode"));
 
   // Load array metadata, if not loaded yet
   if (!metadata_loaded_)
@@ -732,13 +774,13 @@ Status Array::get_metadata_num(uint64_t* num) {
   // Check if array is open
   if (!is_open_)
     return LOG_STATUS(
-        Status::ArrayError("Cannot get number of metadata; Array is not open"));
+        Status_ArrayError("Cannot get number of metadata; Array is not open"));
 
   // Check mode
   if (query_type_ != QueryType::READ)
     return LOG_STATUS(
-        Status::ArrayError("Cannot get number of metadata; Array was "
-                           "not opened in read mode"));
+        Status_ArrayError("Cannot get number of metadata; Array was "
+                          "not opened in read mode"));
 
   // Load array metadata, if not loaded yet
   if (!metadata_loaded_)
@@ -754,18 +796,18 @@ Status Array::has_metadata_key(
   // Check if array is open
   if (!is_open_)
     return LOG_STATUS(
-        Status::ArrayError("Cannot get metadata; Array is not open"));
+        Status_ArrayError("Cannot get metadata; Array is not open"));
 
   // Check mode
   if (query_type_ != QueryType::READ)
     return LOG_STATUS(
-        Status::ArrayError("Cannot get metadata; Array was "
-                           "not opened in read mode"));
+        Status_ArrayError("Cannot get metadata; Array was "
+                          "not opened in read mode"));
 
   // Check if key is null
   if (key == nullptr)
     return LOG_STATUS(
-        Status::ArrayError("Cannot get metadata; Key cannot be null"));
+        Status_ArrayError("Cannot get metadata; Key cannot be null"));
 
   // Load array metadata, if not loaded yet
   if (!metadata_loaded_)
@@ -776,13 +818,13 @@ Status Array::has_metadata_key(
   return Status::Ok();
 }
 
-Metadata* Array::metadata() {
+Metadata* Array::unsafe_metadata() {
   return &metadata_;
 }
 
 Status Array::metadata(Metadata** metadata) {
-  // Load array metadata, if not loaded yet
-  if (!metadata_loaded_)
+  // Load array metadata for array opened for reads, if not loaded yet
+  if (query_type_ == QueryType::READ && !metadata_loaded_)
     RETURN_NOT_OK(load_metadata());
 
   *metadata = &metadata_;
@@ -790,19 +832,22 @@ Status Array::metadata(Metadata** metadata) {
   return Status::Ok();
 }
 
-const NDRange& Array::non_empty_domain() {
+tuple<Status, optional<const NDRange>> Array::non_empty_domain() {
   if (!non_empty_domain_computed_) {
     // Compute non-empty domain
-    compute_non_empty_domain();
+    RETURN_NOT_OK_TUPLE(compute_non_empty_domain(), nullopt);
   }
-  return non_empty_domain_;
+  return {Status::Ok(), non_empty_domain_};
 }
 
 void Array::set_non_empty_domain(const NDRange& non_empty_domain) {
-  std::lock_guard<std::mutex> lock{mtx_};
   non_empty_domain_ = non_empty_domain;
 }
 
+MemoryTracker* Array::memory_tracker() {
+  return &memory_tracker_;
+}
+
 /* ********************************* */
 /*          PRIVATE METHODS          */
 /* ********************************* */
@@ -815,14 +860,14 @@ void Array::clear_last_max_buffer_sizes() {
 
 Status Array::compute_max_buffer_sizes(const void* subarray) {
   // Applicable only to domains where all dimensions have the same type
-  if (!array_schema_latest_->domain()->all_dims_same_type())
+  if (!array_schema_latest_->domain().all_dims_same_type())
     return LOG_STATUS(
-        Status::ArrayError("Cannot compute max buffer sizes; Inapplicable when "
-                           "dimension domains have different types"));
+        Status_ArrayError("Cannot compute max buffer sizes; Inapplicable when "
+                          "dimension domains have different types"));
 
   // Allocate space for max buffer sizes subarray
   auto dim_num = array_schema_latest_->dim_num();
-  auto coord_size = array_schema_latest_->domain()->dimension(0)->coord_size();
+  auto coord_size = array_schema_latest_->domain().dimension(0)->coord_size();
   auto subarray_size = 2 * dim_num * coord_size;
   last_max_buffer_sizes_subarray_.resize(subarray_size);
 
@@ -842,7 +887,7 @@ Status Array::compute_max_buffer_sizes(const void* subarray) {
         std::pair<uint64_t, uint64_t>(0, 0);
     for (unsigned d = 0; d < dim_num; ++d)
       last_max_buffer_sizes_
-          [array_schema_latest_->domain()->dimension(d)->name()] =
+          [array_schema_latest_->domain().dimension(d)->name()] =
               std::pair<uint64_t, uint64_t>(0, 0);
 
     RETURN_NOT_OK(compute_max_buffer_sizes(subarray, &last_max_buffer_sizes_));
@@ -861,10 +906,10 @@ Status Array::compute_max_buffer_sizes(
   if (remote_) {
     auto rest_client = storage_manager_->rest_client();
     if (rest_client == nullptr)
-      return LOG_STATUS(Status::ArrayError(
+      return LOG_STATUS(Status_ArrayError(
           "Cannot get max buffer sizes; remote array with no REST client."));
     return rest_client->get_array_max_buffer_sizes(
-        array_uri_, array_schema_latest_, subarray, buffer_sizes);
+        array_uri_, *(array_schema_latest_.get()), subarray, buffer_sizes);
   }
 
   // Return if there are no metadata
@@ -891,7 +936,7 @@ Status Array::compute_max_buffer_sizes(
 
   // Rectify bound for dense arrays
   if (array_schema_latest_->dense()) {
-    auto cell_num = array_schema_latest_->domain()->cell_num(sub);
+    auto cell_num = array_schema_latest_->domain().cell_num(sub);
     // `cell_num` becomes 0 when `subarray` is huge, leading to a
     // `uint64_t` overflow.
     if (cell_num != 0) {
@@ -910,8 +955,8 @@ Status Array::compute_max_buffer_sizes(
 
   // Rectify bound for sparse arrays with integer domain, without duplicates
   if (!array_schema_latest_->dense() && !array_schema_latest_->allows_dups() &&
-      array_schema_latest_->domain()->all_dims_int()) {
-    auto cell_num = array_schema_latest_->domain()->cell_num(sub);
+      array_schema_latest_->domain().all_dims_int()) {
+    auto cell_num = array_schema_latest_->domain().cell_num(sub);
     // `cell_num` becomes 0 when `subarray` is huge, leading to a
     // `uint64_t` overflow.
     if (cell_num != 0) {
@@ -935,21 +980,17 @@ Status Array::compute_max_buffer_sizes(
 }
 
 Status Array::load_metadata() {
-  std::lock_guard<std::mutex> lock{mtx_};
   if (remote_) {
     auto rest_client = storage_manager_->rest_client();
     if (rest_client == nullptr)
-      return LOG_STATUS(Status::ArrayError(
+      return LOG_STATUS(Status_ArrayError(
           "Cannot load metadata; remote array with no REST client."));
     RETURN_NOT_OK(rest_client->get_array_metadata_from_rest(
         array_uri_, timestamp_start_, timestamp_end_opened_at_, this));
   } else {
+    assert(array_dir_.loaded());
     RETURN_NOT_OK(storage_manager_->load_array_metadata(
-        array_uri_,
-        *encryption_key_,
-        timestamp_start_,
-        timestamp_end_opened_at_,
-        &metadata_));
+        array_dir_, *encryption_key_, &metadata_));
   }
   metadata_loaded_ = true;
   return Status::Ok();
@@ -959,7 +1000,7 @@ Status Array::load_remote_non_empty_domain() {
   if (remote_) {
     auto rest_client = storage_manager_->rest_client();
     if (rest_client == nullptr)
-      return LOG_STATUS(Status::ArrayError(
+      return LOG_STATUS(Status_ArrayError(
           "Cannot load metadata; remote array with no REST client."));
     RETURN_NOT_OK(rest_client->get_array_non_empty_domain(
         this, timestamp_start_, timestamp_end_opened_at_));
@@ -972,7 +1013,6 @@ Status Array::compute_non_empty_domain() {
   if (remote_) {
     RETURN_NOT_OK(load_remote_non_empty_domain());
   } else if (!fragment_metadata_.empty()) {
-    std::lock_guard<std::mutex> lock{mtx_};
     const auto& frag0_dom = fragment_metadata_[0]->non_empty_domain();
     non_empty_domain_.assign(frag0_dom.begin(), frag0_dom.end());
 
@@ -985,13 +1025,13 @@ Status Array::compute_non_empty_domain() {
       // corrupt for these cases we want to check to prevent any segfaults
       // later.
       if (!meta_dom.empty())
-        array_schema_latest_->domain()->expand_ndrange(
+        array_schema_latest_->domain().expand_ndrange(
             meta_dom, &non_empty_domain_);
       else {
         // If the fragment's non-empty domain is indeed empty, lets log it so
         // the user gets a message warning that this fragment might be corrupt
         // Note: LOG_STATUS only prints if TileDB is built in verbose mode.
-        LOG_STATUS(Status::ArrayError(
+        LOG_STATUS(Status_ArrayError(
             "Non empty domain unexpectedly empty for fragment: " +
             fragment_metadata_[j]->fragment_uri().to_string()));
       }
