@@ -120,7 +120,8 @@ void InfoCommand::run() {
 
 void InfoCommand::print_tile_sizes() const {
   stats::Stats stats("");
-  StorageManager sm(&compute_tp_, &io_tp_, &stats, tdb_make_shared(Logger, ""));
+  StorageManager sm(
+      &compute_tp_, &io_tp_, &stats, make_shared<Logger>(HERE(), ""));
   THROW_NOT_OK(sm.init(nullptr));
 
   // Open the array
@@ -131,9 +132,9 @@ void InfoCommand::print_tile_sizes() const {
   EncryptionKey enc_key;
 
   // Compute and report mean persisted tile sizes over all attributes.
-  const auto* schema = array.array_schema_latest();
+  const auto& schema = array.array_schema_latest();
   auto fragment_metadata = array.fragment_metadata();
-  auto attributes = schema->attributes();
+  auto attributes = schema.attributes();
   uint64_t total_persisted_size = 0, total_in_memory_size = 0;
 
   // Helper function for processing each attribute.
@@ -147,16 +148,19 @@ void InfoCommand::print_tile_sizes() const {
       THROW_NOT_OK(f->load_tile_offsets(enc_key, std::move(names)));
       THROW_NOT_OK(f->load_tile_var_sizes(enc_key, name));
       for (uint64_t tile_idx = 0; tile_idx < tile_num; tile_idx++) {
-        uint64_t tile_size = 0;
-        THROW_NOT_OK(f->persisted_tile_size(name, tile_idx, &tile_size));
-        persisted_tile_size += tile_size;
+        auto&& [st, tile_size] = f->persisted_tile_size(name, tile_idx);
+        THROW_NOT_OK(st);
+        persisted_tile_size += *tile_size;
         in_memory_tile_size += f->tile_size(name, tile_idx);
         num_tiles++;
         if (var_size) {
-          THROW_NOT_OK(f->persisted_tile_var_size(name, tile_idx, &tile_size));
-          persisted_tile_size += tile_size;
-          THROW_NOT_OK(f->tile_var_size(name, tile_idx, &tile_size));
-          in_memory_tile_size += tile_size;
+          auto&& [st_var_persisted, tile_size_var_persisted] =
+              f->persisted_tile_var_size(name, tile_idx);
+          THROW_NOT_OK(st_var_persisted);
+          persisted_tile_size += *tile_size_var_persisted;
+          auto&& [st_var, tile_size_var] = f->tile_var_size(name, tile_idx);
+          THROW_NOT_OK(st_var);
+          in_memory_tile_size += *tile_size_var;
           num_tiles++;
         }
       }
@@ -176,11 +180,11 @@ void InfoCommand::print_tile_sizes() const {
   std::cout << "Tile stats (per attribute):" << std::endl;
 
   // Dump info about coords for sparse arrays.
-  if (!schema->dense())
+  if (!schema.dense())
     process_attr(constants::coords, false);
 
   // Dump info about the rest of the attributes
-  for (const auto* attr : attributes)
+  for (const auto& attr : attributes)
     process_attr(attr->name(), attr->var_size());
 
   std::cout << "Sum of attribute persisted size: " << total_persisted_size
@@ -194,7 +198,8 @@ void InfoCommand::print_tile_sizes() const {
 
 void InfoCommand::print_schema_info() const {
   stats::Stats stats("");
-  StorageManager sm(&compute_tp_, &io_tp_, &stats, tdb_make_shared(Logger, ""));
+  StorageManager sm(
+      &compute_tp_, &io_tp_, &stats, make_shared<Logger>(HERE(), ""));
   THROW_NOT_OK(sm.init(nullptr));
 
   // Open the array
@@ -203,7 +208,7 @@ void InfoCommand::print_schema_info() const {
   THROW_NOT_OK(
       array.open(QueryType::READ, EncryptionType::NO_ENCRYPTION, nullptr, 0));
 
-  array.array_schema_latest()->dump(stdout);
+  array.array_schema_latest().dump(stdout);
 
   // Close the array.
   THROW_NOT_OK(array.close());
@@ -211,7 +216,8 @@ void InfoCommand::print_schema_info() const {
 
 void InfoCommand::write_svg_mbrs() const {
   stats::Stats stats("");
-  StorageManager sm(&compute_tp_, &io_tp_, &stats, tdb_make_shared(Logger, ""));
+  StorageManager sm(
+      &compute_tp_, &io_tp_, &stats, make_shared<Logger>(HERE(), ""));
   THROW_NOT_OK(sm.init(nullptr));
 
   // Open the array
@@ -220,8 +226,8 @@ void InfoCommand::write_svg_mbrs() const {
   THROW_NOT_OK(
       array.open(QueryType::READ, EncryptionType::NO_ENCRYPTION, nullptr, 0));
 
-  const auto* schema = array.array_schema_latest();
-  auto dim_num = schema->dim_num();
+  const auto& schema = array.array_schema_latest();
+  auto dim_num = schema.dim_num();
   if (dim_num < 2) {
     THROW_NOT_OK(array.close());
     throw std::runtime_error("SVG MBRs only supported for >1D arrays.");
@@ -236,7 +242,7 @@ void InfoCommand::write_svg_mbrs() const {
   for (const auto& f : fragment_metadata) {
     const auto& mbrs = f->mbrs();
     for (const auto& mbr : mbrs) {
-      auto tup = get_mbr(mbr, schema->domain());
+      auto tup = get_mbr(mbr, schema.domain());
       min_x = std::min(min_x, std::get<0>(tup));
       min_y = std::min(min_y, std::get<1>(tup));
       max_x = std::max(max_x, std::get<0>(tup) + std::get<2>(tup));
@@ -286,7 +292,8 @@ void InfoCommand::write_svg_mbrs() const {
 
 void InfoCommand::write_text_mbrs() const {
   stats::Stats stats("");
-  StorageManager sm(&compute_tp_, &io_tp_, &stats, tdb_make_shared(Logger, ""));
+  StorageManager sm(
+      &compute_tp_, &io_tp_, &stats, make_shared<Logger>(HERE(), ""));
   THROW_NOT_OK(sm.init(nullptr));
 
   // Open the array
@@ -296,15 +303,15 @@ void InfoCommand::write_text_mbrs() const {
       array.open(QueryType::READ, EncryptionType::NO_ENCRYPTION, nullptr, 0));
 
   auto encryption_key = array.encryption_key();
-  const auto* schema = array.array_schema_latest();
-  auto dim_num = schema->dim_num();
+  const auto& schema = array.array_schema_latest();
+  auto dim_num = schema.dim_num();
   auto fragment_metadata = array.fragment_metadata();
   std::stringstream text;
   for (const auto& f : fragment_metadata) {
     f->load_rtree(*encryption_key);
     const auto& mbrs = f->mbrs();
     for (const auto& mbr : mbrs) {
-      auto str_mbr = mbr_to_string(mbr, schema->domain());
+      auto str_mbr = mbr_to_string(mbr, schema.domain());
       for (unsigned i = 0; i < dim_num; i++) {
         text << str_mbr[2 * i + 0] << "," << str_mbr[2 * i + 1];
         if (i < dim_num - 1)
@@ -326,12 +333,12 @@ void InfoCommand::write_text_mbrs() const {
 }
 
 std::tuple<double, double, double, double> InfoCommand::get_mbr(
-    const NDRange& mbr, const Domain* domain) const {
-  assert(domain->dim_num() == 2);
+    const NDRange& mbr, const tiledb::sm::Domain& domain) const {
+  assert(domain.dim_num() == 2);
   double x, y, width, height;
 
   // First dimension
-  auto d1_type = domain->dimension(0)->type();
+  auto d1_type = domain.dimension(0)->type();
   switch (d1_type) {
     case Datatype::INT8:
       y = static_cast<const int8_t*>(mbr[0].data())[0];
@@ -404,7 +411,7 @@ std::tuple<double, double, double, double> InfoCommand::get_mbr(
   }
 
   // Second dimension
-  auto d2_type = domain->dimension(1)->type();
+  auto d2_type = domain.dimension(1)->type();
   switch (d2_type) {
     case Datatype::INT8:
       x = static_cast<const int8_t*>(mbr[1].data())[0];
@@ -481,7 +488,7 @@ std::tuple<double, double, double, double> InfoCommand::get_mbr(
 
 // Works only for fixed-sized coordinates
 std::vector<std::string> InfoCommand::mbr_to_string(
-    const NDRange& mbr, const Domain* domain) const {
+    const NDRange& mbr, const tiledb::sm::Domain& domain) const {
   std::vector<std::string> result;
   const int8_t* r8;
   const uint8_t* ru8;
@@ -493,9 +500,9 @@ std::vector<std::string> InfoCommand::mbr_to_string(
   const uint64_t* ru64;
   const float* rf32;
   const double* rf64;
-  auto dim_num = domain->dim_num();
+  auto dim_num = domain.dim_num();
   for (unsigned d = 0; d < dim_num; d++) {
-    auto type = domain->dimension(d)->type();
+    auto type = domain.dimension(d)->type();
     switch (type) {
       case sm::Datatype::STRING_ASCII:
         result.push_back(std::string(mbr[d].start_str()));
